package com.bcxin.message.scheduled.message;

import cn.hutool.core.date.DateUtil;
import com.bcxin.message.entity.MsgSendLog;
import lombok.extern.slf4j.Slf4j;
import org.quartz.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.util.Calendar;
import java.util.Date;

/**
 * description：发送重试任务调度
 * author：linchunpeng
 * date：2023/11/22
 */
@Slf4j
@Component
public class SendRetryScheduler {

    @Autowired
    private Scheduler scheduler;
    
    //失败重试间隔 秒
    @Value("${message.retry.interval}")
    private Integer retryInterval;

    /**
     * description：调度发送重试任务
     * author：linchunpeng
     * date：2023/11/22
     */
    public void scheduleSendRetry(MsgSendLog msgSendLog) {
        try {
            log.info("调度发送重试任务，requestId：{}", msgSendLog.getRequestId());
            JobDetail jobDetail = createSendRetryJobDetail(msgSendLog);
            Trigger trigger = createSendRetryTrigger(msgSendLog);
            if (trigger != null) {
                log.info("放入调度线程，requestId：{}", msgSendLog.getRequestId());
                scheduler.scheduleJob(jobDetail, trigger);
            }
        } catch (SchedulerException e) {
            e.printStackTrace();
        }
    }

    /**
     * description：重新调度发送重试任务
     * author：linchunpeng
     * date：2023/11/22
     */
    public void reScheduleSendRetry(MsgSendLog msgSendLog) {
        try {
            log.info("重新调度发送重试任务，requestId：{}", msgSendLog.getRequestId());
            JobDetail jobDetail = scheduler.getJobDetail(this.makeSendRetryJobKey(msgSendLog.getId()));
            Trigger trigger = TriggerBuilder.newTrigger()
                    .forJob(jobDetail)
                    .startAt(DateUtil.offsetSecond(new Date(), retryInterval))
                    .build();
            scheduler.scheduleJob(trigger);
        } catch (SchedulerException e) {
            e.printStackTrace();
        }
    }

    /**
     * description：创建发送重试任务
     * author：linchunpeng
     * date：2023/11/22
     */
    private JobDetail createSendRetryJobDetail(MsgSendLog msgSendLog) {
        log.info("创建发送重试任务，requestId：{}", msgSendLog.getRequestId());
        return JobBuilder.newJob(SendRetryJob.class)
                .withIdentity(this.makeSendRetryJobKey(msgSendLog.getId()))
                .usingJobData("jobId", msgSendLog.getId())
                .storeDurably()
                .build();
    }

    /**
     * description：创建发送重试任务触发器
     * author：linchunpeng
     * date：2023/11/22
     */
    private Trigger createSendRetryTrigger(MsgSendLog msgSendLog) {
        log.info("创建发送重试任务触发器，requestId：{}", msgSendLog.getRequestId());
        Calendar startTime = Calendar.getInstance();
        startTime.setTime(new Date());
        startTime.add(Calendar.SECOND, retryInterval);
        return TriggerBuilder.newTrigger()
                .withIdentity(this.makeSendRetryTriggerKey(msgSendLog.getId()))
                .startAt(startTime.getTime())
                .build();
    }

    /**
     * description：取消发送重试任务
     * author：linchunpeng
     * date：2023/11/22
     */
    public void cancelSendRetryJob(MsgSendLog msgSendLog) {
        try {
            log.info("取消发送重试任务，requestId：{}", msgSendLog.getRequestId());
            scheduler.pauseTrigger(this.makeSendRetryTriggerKey(msgSendLog.getId()));
            scheduler.pauseJob(this.makeSendRetryJobKey(msgSendLog.getId()));
            scheduler.deleteJob(this.makeSendRetryJobKey(msgSendLog.getId()));
        } catch (SchedulerException e) {
            e.printStackTrace();
        }
    }

    /**
     * description：生成发送重试任务key
     * author：linchunpeng
     * date：2023/11/22
     */
    private JobKey makeSendRetryJobKey(Long jobId) {
        return JobKey.jobKey(jobId.toString(), "SEND_RETRY");
    }

    /**
     * description：生成发送重试任务触发器key
     * author：linchunpeng
     * date：2023/11/22
     */
    private TriggerKey makeSendRetryTriggerKey(Long jobId) {
        return TriggerKey.triggerKey(jobId.toString(), "SEND_RETRY");
    }

}
