package com.bcxin.message.service;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.bcxin.message.common.utils.IdGeneratorSnowflake;
import com.bcxin.message.dao.mapper.MsgSendLogMapper;
import com.bcxin.message.dtos.request.SendRobotMessageRequest;
import com.bcxin.message.entity.MsgSendLog;
import com.bcxin.message.scheduled.message.SendRetryScheduler;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Date;

/**
 * description：消息中心发送日志表 service
 * author：linchunpeng
 * date：2023/11/17
 */
@Slf4j
@Service
public class MsgSendLogService extends ServiceImpl<MsgSendLogMapper, MsgSendLog> {

    @Autowired
    private MsgSendLogMapper dao;

    @Autowired
    private IdGeneratorSnowflake snowflake;

    @Autowired
    private SendRetryScheduler sendRetryScheduler;

    //失败重试次数
    @Value("${message.retry.count}")
    private Integer retryCount;

    /**
     * description：记录发送日志
     * author：linchunpeng
     * date：2023/11/21
     */
    @Transactional
    public void record(SendRobotMessageRequest request, String sendStrategy, Integer sendSuccess, String sendResult) {
        MsgSendLog msgSendLog;
        int sendRetryCount = 0;
        MsgSendLog exist = this.findByRequestId(request.getRequestId());
        if (exist != null) {
            msgSendLog = exist;
            sendRetryCount = exist.getSendRetryCount() + 1;
        } else {
            msgSendLog = new MsgSendLog();
            msgSendLog.setId(snowflake.snowflakeId());
            msgSendLog.setRequestId(request.getRequestId());
            msgSendLog.setAppid(request.getAppid());
            msgSendLog.setSender(request.getSender());
            msgSendLog.setSenderType(request.getSenderType());
            msgSendLog.setSendStrategy(sendStrategy);
            msgSendLog.setSendParam(JSONObject.toJSONString(request));
            msgSendLog.setCreateTime(new Date());
        }
        msgSendLog.setSendSuccess(sendSuccess);
        msgSendLog.setSendResult(sendResult);
        msgSendLog.setSendRetryCount(sendRetryCount);

        String resultMsg = sendSuccess == 1 ? "成功" : "失败";
        if (sendRetryCount == 0) {
            msgSendLog.setRemark(String.format("发送消息%s", resultMsg));
        } else {
            msgSendLog.setRemark(String.format("发送重试，第%s次发送消息%s", sendRetryCount, resultMsg));
        }
        msgSendLog.setUpdateTime(new Date());
        if (exist != null) {
            this.updateById(msgSendLog);
        } else {
            this.saveOrUpdate(msgSendLog);
        }

        //如果是失败，需要判断重试次数 < 规定次数，如果小于，则调度重试机制
        if (sendSuccess == 0 && sendRetryCount <= retryCount) {
            if (sendRetryCount == 0) {
                //第一次，新增调度
                sendRetryScheduler.scheduleSendRetry(msgSendLog);
            } else {
                //不是第一次，重启调度
                sendRetryScheduler.reScheduleSendRetry(msgSendLog);
            }
        }
    }

    /**
     * description：通过requestId查询记录
     * author：linchunpeng
     * date：2023/11/22
     */
    public MsgSendLog findByRequestId(String requestId) {
        LambdaQueryWrapper<MsgSendLog> lqw = Wrappers.lambdaQuery();
        lqw.eq(MsgSendLog::getRequestId, requestId);
        return dao.selectOne(lqw);
    }
}
